@@ -1,6 +1,7 @@
 /* actions.c
  *
- * Copyright 2002 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright 2002 Lutz Mueller <lutz@users.sourceforge.net>
+ * Copyright 2016 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -915,7 +916,7 @@ print_version_action (GPParams __unused__ *p)
 		  "\n"
 		  "This version of gphoto2 is using the following software versions and options:\n"),
 		VERSION,
-		2015, /* year of release! */
+		2016, /* year of release! */
 		port_message
 		);
 
@@ -1543,6 +1544,13 @@ static int
 _find_widget_by_name (GPParams *p, const char *name, CameraWidget **child, CameraWidget **rootconfig) {
 	int	ret;
 
+	*rootconfig = NULL;
+	ret = gp_camera_get_single_config (p->camera, name, child, p->context);
+	if (ret == GP_OK) {
+		*rootconfig = *child;
+		return GP_OK;
+	}
+
 	ret = gp_camera_get_config (p->camera, rootconfig, p->context);
 	if (ret != GP_OK) return ret;
 	ret = gp_widget_get_child_by_name (*rootconfig, name, child);
@@ -1571,19 +1579,19 @@ _find_widget_by_name (GPParams *p, const char *name, CameraWidget **child, Camer
 			if (ret != GP_OK)
 				break;
 			*child = tmp;
-			if (!s) /* end of path */
-				break;
+			if (!s) {
+				/* end of path */
+				free (newname);
+				return GP_OK;
+			}
 			part = s+1;
 			while (part[0] == '/')
 				part++;
 		}
-		if (s) { /* if we have stuff left over, we failed */
-			gp_context_error (p->context, _("%s not found in configuration tree."), newname);
-			free (newname);
-			gp_widget_free (*rootconfig);
-			return GP_ERROR;
-		}
+		gp_context_error (p->context, _("%s not found in configuration tree."), newname);
 		free (newname);
+		gp_widget_free (*rootconfig);
+		return GP_ERROR;
 	}
 	return GP_OK;
 }
@@ -1897,7 +1905,10 @@ set_config_action (GPParams *p, const char *name, const char *value) {
 		break;
 	}
 	if (ret == GP_OK) {
-		ret = gp_camera_set_config (p->camera, rootconfig, p->context);
+		if (child == rootconfig)
+			ret = gp_camera_set_single_config (p->camera, name, child, p->context);
+		else
+			ret = gp_camera_set_config (p->camera, rootconfig, p->context);
 		if (ret != GP_OK)
 			gp_context_error (p->context, _("Failed to set new configuration value %s for configuration entry %s."), value, name);
 	}
@@ -1965,7 +1976,10 @@ set_config_index_action (GPParams *p, const char *name, const char *value) {
 		break;
 	}
 	if (ret == GP_OK) {
-		ret = gp_camera_set_config (p->camera, rootconfig, p->context);
+		if (child == rootconfig)
+			ret = gp_camera_set_single_config (p->camera, name, child, p->context);
+		else
+			ret = gp_camera_set_config (p->camera, rootconfig, p->context);
 		if (ret != GP_OK)
 			gp_context_error (p->context, _("Failed to set new configuration value %s for configuration entry %s."), value, name);
 	}
@@ -2105,7 +2119,10 @@ set_config_value_action (GPParams *p, const char *name, const char *value) {
 		break;
 	}
 	if (ret == GP_OK) {
-		ret = gp_camera_set_config (p->camera, rootconfig, p->context);
+		if (child == rootconfig)
+			ret = gp_camera_set_single_config (p->camera, name, child, p->context);
+		else
+			ret = gp_camera_set_config (p->camera, rootconfig, p->context);
 		if (ret != GP_OK)
 			gp_context_error (p->context, _("Failed to set new configuration value %s for configuration entry %s."), value, name);
 	}
